
From: Jan Kara <jack@suse.cz>

Fix possible false assertion failure in log_do_checkpoint().  We might fail
to detect that we actually made a progress when cleaning up the checkpoint
lists if we don't retry after writing something to disk.  The patch was
confirmed to fix observed assertion failures for several users.

When we flushed some buffers we need to retry scanning the list.
Otherwise we can fail to detect our progress.

Signed-off-by: Jan Kara <jack@suse.cz>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 fs/jbd/checkpoint.c |    4 +++-
 1 files changed, 3 insertions(+), 1 deletion(-)

diff -puN fs/jbd/checkpoint.c~ext3-fix-log_do_checkpoint-assertion-failure fs/jbd/checkpoint.c
--- 25/fs/jbd/checkpoint.c~ext3-fix-log_do_checkpoint-assertion-failure	2005-06-01 00:55:16.000000000 -0700
+++ 25-akpm/fs/jbd/checkpoint.c	2005-06-01 00:55:16.000000000 -0700
@@ -339,8 +339,10 @@ int log_do_checkpoint(journal_t *journal
 			}
 		} while (jh != last_jh && !retry);
 
-		if (batch_count)
+		if (batch_count) {
 			__flush_batch(journal, bhs, &batch_count);
+			retry = 1;
+		}
 
 		/*
 		 * If someone cleaned up this transaction while we slept, we're
_
